package mmj;

import java.awt.*;
import java.util.*;

public class Executer extends MMJRuntime {
  public static final int FLAG_REAL = 1;
  public static final int FLAG_VIRTUAL = 2;
  public static final int FLAG_GRAPHICS = 3;

  Thread thread;
  int FLAG;
  final Object WAIT = new Object();
  final Object STOP = new Object();
  int flag;
  boolean wait, step, stop;

  Expression current;
  Environment env;
  Vector obs = new Vector();

  public Executer(Environment e) {
	this.env = e;
  }

  private void init0() {
	wait = false;
	step = false;
	stop = false;
	flag = 0;
	init();
  }
  private void init() {
	FLAG = 0;
	env.init();
	current = env.getFirstExpression();
	super.clearCanvas();
  }

  public void pause() {
	synchronized (WAIT) {
	  wait = true;
	}
  }
  public void play() {
	synchronized (WAIT) {
	  wait = false;
	  WAIT.notify();
	}
  }
  public void stop() {
	if (thread == null) return;
	synchronized (STOP) {
	  stop = true;
	}
  }

  final Executer exec = this;
  private Thread getThread() {
	if (thread != null) return(thread);
	stop = false;
	return(new Thread() {
	  public void run() {
		//		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		//		Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
		Thread.currentThread().setPriority(Thread.NORM_PRIORITY-1);
		env.clearTimeCount();
		
		while (current != null) {
		  FLAG = 0;
		  env.continueTimeCount();
		  current.evaluate(env, exec);
		  current = exec.current.next(env);
		  env.pauseTimeCount();

		  synchronized (STOP) {
			if (stop) return;
		  }
		  synchronized (WAIT) {
			if (step || (FLAG & flag) != 0) pause();
			
			if (wait) {
			  try {
				inform();
				WAIT.wait();
			  } catch (InterruptedException e) { }
			}
		  }
		}
	  }
	});
  }

  public void interpret() {
	if (thread == null || ! thread.isAlive()) {
	  init();
	  thread = getThread();
	  thread.start();
	} else {
	  play();
	}
  }
  public void execute() {
	flag = 0;
	step = false;
	interpret();
  }
  public void executeNext() {
	flag = 0;
	step = true;
	interpret();
  }
  public void executeNext(int f) {
	flag = f;
	step = false;
	interpret();
  }


  public synchronized void addObserver(ExecutionObserver o) {
	if (! obs.contains(o)) {
	  obs.addElement(o);
	}
  }
  public synchronized void removeObserver(ExecutionObserver o) {
	if (obs.contains(o)) {
	  obs.removeElement(o);
	}
  }
  public synchronized void inform() {
	for (Enumeration e = obs.elements(); e.hasMoreElements();) {
	  ExecutionObserver o = (ExecutionObserver) e.nextElement();
	  o.update(current);
	}
  }

  public void setSize(int w, int h) {
	super.setSize(w,h);
	FLAG = FLAG_REAL;
  }
  public void drawLine(int x, int y, int x2, int y2, Color c) {
	super.drawLine(x,y,x2,y2,c);
	FLAG = FLAG_VIRTUAL;
  }
  public void fillRect(int x, int y, int w, int h, Color c) {
	super.fillRect(x,y,w,h,c);
	FLAG = FLAG_VIRTUAL;
  }
  public void copyArea(int x, int y, int w, int h, int dx, int dy) {
	super.copyArea(x,y,w,h,dx,dy);
	FLAG = FLAG_VIRTUAL;
  }
  public void drawImage(int id, int x, int y) {
	super.drawImage(id,x,y);
	FLAG = FLAG_VIRTUAL;
  }
  public void closeWindow(int id) {
	super.closeWindow(id);
  }
  public void openWindow(int id, int x, int y, int w, int h) {
	super.openWindow(id,x,y,w,h);
  }
  public void setCopyFrom(int id, int x, int y) {
	super.setCopyFrom(id,x,y);
  }
  public void transport() {
	super.transport();
	FLAG = FLAG_REAL;
  }
}
