package mmj;

import mmj.util.BufferedCanvas;
import java.awt.*;

class MMJCanvas {
  private BufferedCanvas rComp, vComp;

  private Point fromPoint[];
  private Rectangle geometry[];

  MMJCanvas() {
	rComp = new BufferedCanvas();
	rComp.setName("RealCanvas");

	vComp = new BufferedCanvas();
	vComp.setName("VirtualCanvas");

	init();
  }
  Component getRealCanvas() { return rComp; }
  Component getVirtualCanvas() { return vComp; }

  public void init() {
	rComp.setBackground(Color.black);
	rComp.setSize(320,200);
	vComp.setBackground(Color.black);
	vComp.setSize(640,800);

	geometry = new Rectangle[8];
	fromPoint = new Point[8];
	for (int i=0; i<fromPoint.length; i++) {
	  fromPoint[i] = new Point(0,0);
	}
  }

  void setSize(int w, int h) {
	rComp.setSize(w,h);
  }

  void drawLine(int x1, int y1, int x2, int y2, Color c) {
	Graphics g = vComp.getBufferedGraphics();
	if (g != null) {
	  if (c == null) c = vComp.getBackground();
	  g.setColor(c);
	  g.drawLine(x1,y1,x2,y2);
	  vComp.repaintNow();
	  g.dispose();
	}
  }
  void fillRect(int x, int y, int w, int h, Color c) {
	Graphics g = vComp.getBufferedGraphics();
	if (g != null) {
	  if (c == null) c = vComp.getBackground();
	  g.setColor(c);
	  g.fillRect(x,y,w,h);
	  vComp.repaintNow();
	  g.dispose();
	}
  }
  void copyArea(int x, int y, int w, int h, int dx, int dy) {
	Graphics g = vComp.getBufferedGraphics();
	if (g != null) {
	  g.copyArea(x,y,w,h,dx,dy);
	  vComp.repaintNow();
	  g.dispose();
	}
  }
  void drawImage(Image im, int x, int y) {
	Graphics g = vComp.getBufferedGraphics();
	if (g != null) {
	  g.drawImage(im, x, y, null);
	  g.dispose();
	  vComp.repaintNow();
	}
  }

  void closeWindow(int id) {
	geometry[id] = null;
  }
  void openWindow(int id, int x, int y, int w, int h) {
	if (w == 0 && h == 0) {
	  closeWindow(id);
	} else {
	  geometry[id] = new Rectangle(x,y,w,h);
	}
  }
  void setCopyFrom(int id, int x, int y) {
	fromPoint[id] = new Point(x,y);
  }
  void transport() {
	Graphics g = rComp.getBufferedGraphics();
	if (g == null) return;
	for (int i=0; i<geometry.length; i++) {
	  if (geometry[i] != null) {
		g.setClip(geometry[i].x, geometry[i].y,
				  geometry[i].width, geometry[i].height);
		g.drawImage(vComp.getBuffer(),
					geometry[i].x - fromPoint[i].x,
					geometry[i].y - fromPoint[i].y,
					null);
	  }
	}
	rComp.repaintNow();
	g.dispose();
  }
}
