package mmj;

import java.awt.*;
import java.awt.image.ColorModel;
import java.io.InputStream;
import java.io.IOException;
import java.util.Vector;

public class MMJRuntime {
  Image image[];
  Point moveTo[];

  ColorModel palette;
  protected MMJCanvas canvas;

  public MMJRuntime() {
	canvas = new MMJCanvas();
  }

  public synchronized Component getVirtualCanvas() {
	return(canvas.getVirtualCanvas());
  }
  public synchronized Component getRealCanvas() {
	return(canvas.getRealCanvas());
  }
  public void clearCanvas() {
	canvas.init();
  }

  public void setSize(int w, int h) {
	canvas.setSize(w,h);
  }
  public void drawLine(int x, int y, int x2, int y2, Color c) {
	canvas.drawLine(x,y,x2,y2,c);
  }
  public void fillRect(int x, int y, int w, int h, Color c) {
	canvas.fillRect(x,y,w,h,c);
  }
  public void copyArea(int x, int y, int w, int h, int dx, int dy) {
	canvas.copyArea(x,y,w,h,dx,dy);
  }
  public void orCopyArea(int x, int y, int w, int h, int dx, int dy) {
	System.err.println("\"or move\" is not supported.");
  }
  public void zoom(int x1, int y1, int w1, int h1,
				   int x2, int y2, int w2, int h2, Color trans) {
	System.err.println("\"zoom\" is not supported.");
  }
  public void drawImage(int id, int x, int y) {
	if (image != null) {
	  if (moveTo[id] != null) {
		x += moveTo[id].x;
		y += moveTo[id].y;
	  }
	  canvas.drawImage(image[id], x, y);
	}
  }

  public void closeWindow(int id) {
	canvas.closeWindow(id);
  }
  public void openWindow(int id, int x, int y, int w, int h) {
	canvas.openWindow(id,x,y,w,h);
  }
  public void setCopyFrom(int id, int x, int y) {
	canvas.setCopyFrom(id,x,y);
  }
  public void transport() {
	canvas.transport();
  }

  public synchronized void setImage(int id, Image im) {
	if (image == null) {
	  image = new Image[id+1];
	  moveTo = new Point[id+1];
	} else if (image.length <= id) {
	  Image[] i = new Image[id+1];
	  System.arraycopy(image, 0, i, 0, image.length);
	  image = i;

	  Point[] p = new Point[id+1];
	  System.arraycopy(moveTo, 0, p, 0, moveTo.length);
	  moveTo = p;
	}
	if (image[id] != null)
	  image[id].flush();
	image[id] = im;
  }
  public synchronized void setImage(Image[] im) {
	if (image != null) {
	  for (int i=0; i<image.length; i++)
		if (image[i] != null) image[i].flush();
	}
	if (im == null) {
	  image = null;
	} else {
	  image = new Image[im.length];
	  System.arraycopy(im,0, image,0, im.length);
	  moveTo = new Point[image.length];
	}
  }
  public synchronized void setImageMove(int id, Point a) {
	if (moveTo == null || moveTo.length <= id) return;
	moveTo[id] = a;
  }

  /*
  public Image getImage(int id) {
	return(image[id]);
  }
  */
  public int getImageCount() {
	return((image==null) ? 0 : image.length);
  }

  public void setPCM(int id, PCM pcm) { }
  public void setPCM(PCM[] pcm) { }
  public int getPCMCount() { return 0; }

  public void setPalette(ColorModel palette) { this.palette = palette; }
  public Color getColor(int id) { return(new Color(palette.getRGB(id))); }
  public int getPaletteCount() { return 1; }

  public boolean isGModeAvailable(int i) { return true; }
}
