package mmj.mmasl;

class ByteArray {
  private byte array[];
  private int pt;
  private boolean half;
  private int halfPt;
  
  void seekPt(int i) {
	pt += i;
  }
  
  void add(byte b) {
	if (pt >= array.length) {
	  byte ba[] = new byte[pt * 2];
	  System.arraycopy(array,0,ba,0,array.length);
	  array = ba;
	}
	array[pt++] = b;
  }
  
  void add(int i) {
	int j = i & 0xFF;
	add((byte)( (j>127) ? j-256 : j ));
  }

  void addShort(int i) {
	int j = ((short)i) >>> 8;
	add((byte)( (j>127) ? j-256 : j ));
	j = i & 0xFF;
	add((byte)( (j>127) ? j-256 : j ));
  }
  
  void add(byte b[]) {
	if (pt+b.length-1 >= array.length) {
	  byte ba[] = new byte[array.length * 2 + b.length];
	  for (int i=0; i<pt; i++) {
	    ba[i] = array[i];
	  }
	  array = ba;
	}
	for (int i=0; i<b.length; i++) {
	  array[pt++] = b[i];
	}
  }
  
  synchronized void setHalf(boolean b) {
	half = b;
  }
  
  boolean eof() {
	return(pt >= array.length);
  }
  
  void last() {
	pt = array.length;
  }
  
  ByteArray(byte ar[], int i) {
	this.array = ar;
	this.pt = i;
	this.half = false;
  }
  
  int size() {
	return(array.length);
  }
  
  ByteArray(byte ar[]) {
	this(ar, 0);
  }
  
  ByteArray(int i) {
	this(new byte[i], 0);
  }
  
  byte[] getArray() {
	return(this.array);
  }
  
  int getPointer() {
	return(this.pt);
  }
  
  void skip(int i) {
	int newPt = pt + i;
	pt = newPt;
  }
  
  void set(int i) {
	int newPt = i;
	pt = newPt;
  }
  
  void setPt(int i) {
	set(i);
  }
  
  byte getByte() {
	return(array[pt++]);
  }
  
  short get8() {
	byte b = getByte();
	int a = (b>=0) ? b : b+256;
	return((short)a);
  }
  
  int get16() {
	return((get8() | get8() << 8));
  }
  
  short getShort() {
	int i = (get8() << 8 | get8());
	return( (short)( (i > 32767) ? i-65536 : i) );
  }
  
  byte getLo() {
	halfPt = pt;
	int a = array[pt++] & 0x0F;
	return((byte)a);
  }
  
  byte get4() {
	half = (half == false);
	if (half) {
	  return(getLo());
	}
	else {
	  return(getHi(halfPt));
	}
  }
  
  byte getHi(int i) {
	byte b = getByte(i);
	int a = ((b<0) ? b+256 : b) >>> 4;
	return((byte)a);
  }
  
  byte getByte(int i) {
	return(array[i]);
  }
}
