package mmj.mmasl;

import java.util.*;
import java.awt.*;
import java.awt.image.*;

public class NAKExpander implements ImageObserver, Runnable {
  static int DEBUG = 0;
  static boolean MULTI_PALETTE = false;

  ColorModel[] pal;
  NAK[] nak, unexpand;
  Image[][] ret;
  Point imagePoint[];
  boolean loaded[][];
  int loadcount;
  int naksize, palsize;
  Vector observer = new Vector();
  Toolkit tk;
  Hashtable nakid;

  public NAKExpander(MMS mm) {
	this(mm.nak, mm.pal);
  }
  public NAKExpander(NAK n[], ColorModel c[]) {
	tk = Toolkit.getDefaultToolkit();
	if (c != null && c.length > 0) {
	  if (MULTI_PALETTE) {
		pal = c;
	  } else {
		pal = new ColorModel[1];
		pal[0] = c[0];
	  }
	} else {
	  pal = new ColorModel[1];
	  pal[0] = ColorModel.getRGBdefault();
	}

	nak = n;
	//	nak = new NAK[6]; //for debug
	//	System.arraycopy(n,0,nak,0,6);

	naksize = nak.length;
	palsize = pal.length;

	unexpand = new NAK[nak.length];
	System.arraycopy(nak,0, unexpand,0, nak.length);

	ret = new Image[naksize][];
	imagePoint = new Point[naksize];
	loaded = new boolean[naksize][];
	for (int i=0; i<ret.length; i++) {
	  ret[i] = new Image[palsize];
	  imagePoint[i] = new Point(nak[i].x0, nak[i].y0);
	  loaded[i] = new boolean[palsize];
	  for (int j=0; j<loaded[i].length; j++)
		loaded[i][j] = false;
	}
	loadcount = 0;
  }

  public static void usePalette(boolean b) {
	MULTI_PALETTE = b;
  }
  public int getNAKSize() {
	return naksize;
  }
  public int getPALSize() {
	return palsize;
  }

  int getID(NAK n) {
	for (int i=0; i<nak.length; i++) {
	  if (nak[i] == n) return i;
	}
	return -1;
  }

  public Image getImage(int n, int c) {
	return(ret[n][c]);
  }
  public Image[] getImages(int n) {
	return(ret[n]);
  }
  public Image[][] getImages() {
	return(ret);
  }
  public Point getPoint(int i) {
	return(imagePoint[i]);
  }
  public Point[] getPoints() {
	return(imagePoint);
  }

  public synchronized void addObserver(NAKObserver o) {
	if (! observer.contains(o))
	  observer.addElement(o);
  }
  public synchronized void removeObserver(NAKObserver o) {
	observer.removeElement(o);
  }

  Object MEMORY = new Object();
  public void run() {
	Thread thread = Thread.currentThread();
	thread.setPriority(Thread.MIN_PRIORITY);
	System.gc();
	for (int i=0; i<unexpand.length; i++) {
	  if (unexpand[i] == null) continue;
	  synchronized (MEMORY) {
		try {
		  if (DEBUG != 0)
			System.out.println("expand start for " + (i+1) + "/" + unexpand.length);
		  expand(unexpand[i]);
		  MEMORY.wait();
		  unexpand[i] = null;
		} catch (OutOfMemoryError ex) {
		  //			ex.printStackTrace();
		  for (Enumeration e=observer.elements(); e.hasMoreElements();) {
			NAKObserver obsver = (NAKObserver) e.nextElement();
			obsver.death(ex);
		  }
		  throw(new ThreadDeath());
		} catch (InterruptedException e) {
		}
	  }
	}
  }
  private void expand(NAK nak0) {
	final NAK nak = nak0;
	final NAKExpander parent = this;
	(new Thread() {
	  public void run() {
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		nak.expand(); // allocate memory, then expanding.
		parent.expanded(nak);
	  }
	}).start();
  }

  void expanded(NAK n) {
	Hashtable prop = new Hashtable();
	int i = getID(n);
	prop.put("nak", new Integer(i));
	for (int c=0; c<palsize; c++) {
	  prop.put("pal", new Integer(c));
	  n.setProperty(prop);
	  n.setColorModel(pal[c]);
	  ret[i][c] = tk.createImage(n);
	  tk.prepareImage(ret[i][c], n.width, n.height, this);
	}
  }

  void completeImage(int n, int c) {
	loaded[n][c] = true;
	for (int i=0; i<palsize; i++) {
	  if (!loaded[n][i]) return;
	}
	synchronized (MEMORY) {
	  nak[n].free();
	  nak[n] = null;
	  loadcount++;
	  System.gc();
	  MEMORY.notify();
	}

	for (Enumeration e=observer.elements(); e.hasMoreElements();) {
	  NAKObserver obsver = (NAKObserver) e.nextElement();
	  obsver.NAKUpdate(n+1, loadcount, naksize, this);
	}
  }

  public boolean imageUpdate(Image image, int f, int x, int y, int w, int h) {
	if ((f & ERROR) != 0) {
	  System.out.println("Image expand error. ignore.");
	  f = ALLBITS;
	}
	if ((f & (ALLBITS | ABORT)) ==  0) {
	  return(true);
	}
	Image im = ((f & ABORT) == 0) ? image : null;
	for (int j=0; j<naksize; j++) {
	  for (int i=0; i<palsize; i++) {
		if (ret[j][i] == im) {
		  completeImage(j,i);
		  return false;
		}
	  }
	}
	return false;
  }	

  public static void setDebugLevel(int i) {
	DEBUG = i;
  }
}
