package mmj.util;

import java.awt.*;

/*
 * If this is subclass of Component, Window.pack() is strange.
 */
public class BufferedCanvas extends Canvas {
  private Image buffer;
  private final Object BUFFER = new Object();

  public BufferedCanvas() {
  }
  public Image getBuffer() {
	synchronized (BUFFER) {
	  return(buffer);
	}
  }

  // for deprecated method getPeer()
  protected boolean hasPeer() {
	return(getPeer() != null);
  }
  public void addNotify() {
	super.addNotify();
	if (buffer == null) {
	  synchronized (BUFFER) {
		refreshBuffer();
	  }
	}
  }

  private void refreshBuffer() {
	//	if (! hasPeer()) return;
	int w = getSize().width;
	int h = getSize().height;
	if (w <= 0 || h <= 0) return;

	Image next = ImageFactory.create(w, h);
	Graphics g = next.getGraphics();
	if (g == null) return;

	g.setColor(getBackground());
	g.fillRect(0, 0, w, h);
	if (buffer != null) {
	  g.drawImage(buffer, 0, 0, null);
	  buffer.flush();
	}
	synchronized (BUFFER) {
	  buffer = next;
	}
  }

  public void reshape(int x, int y, int w, int h) {
	boolean larger = (getSize().width < w || getSize().height < h);
	super.reshape(x,y,w,h);
	if (larger) refreshBuffer();
  }

  public void clear() {
	synchronized (BUFFER) {
	  Graphics g = buffer.getGraphics();
	  if (g != null) {
		g.setColor(getBackground());
		g.fillRect(0,0, getSize().width, getSize().height);
		g.dispose();
	  }
	}
  }
	
  public void repaintNow() {
	Graphics g = super.getGraphics();
	if (g != null) {
	  paint(g);
	  g.dispose();
	}
  }
  public void update(Graphics g) {
	paint(g);
  }
  public void paint(Graphics g) {
	if (g == null) return;
	if (buffer == null)
	  refreshBuffer();

	if (buffer != null) {
	  g.drawImage(buffer, 0, 0, null);
	} else {
	  g.setColor(getBackground());
	  g.fillRect(0,0, getSize().width, getSize().height);
	}
  }
  public Graphics getBufferedGraphics() {
	Image im = getBuffer();
	if (im == null) return null;
	Graphics g = im.getGraphics();
    return(g); 
  }
}
