package mmj.util;

import java.applet.Applet;
import java.io.*;
import java.net.URL;
import java.util.zip.*;

public class Extension {
  final String EXTENSION;
  public Extension(String s) {
	this.EXTENSION = s;
  }

  public static boolean isMatch(String name, String ext) {
	if (name.length() < ext.length()) return false;
	return(name.substring(name.length()-ext.length()).equals(ext));
  }

  private InputStream getInputStream(InputStream is, String file) throws Exception {
	if (isMatch(file, EXTENSION)) 
	  return(is);
	else if (isMatch(file, ".zip")) {
	  ZipInputStream zis = new ZipInputStream(is);
	  for (ZipEntry ent = zis.getNextEntry();
		   ent != null;
		   ent = zis.getNextEntry()) {
		String name = ent.getName();
		if (isMatch(name, EXTENSION))
		  return(zis);
	  }
	  return(null);
	} else if (isMatch(file, ".gz")) {
	  return(new GZIPInputStream(is));
	} else {
	  return(null);
	}
  }

  public InputStream fromFile(String file) throws Exception {
	InputStream is = new FileInputStream(file);
	return(getInputStream(is, file.toLowerCase()));
  }

  public InputStream fromURL(Applet app, String url) throws Exception {
	InputStream is = URLtoIS(app, url);
	return(getInputStream(is, url.toLowerCase()));
  }

  private static InputStream URLtoIS(Applet app, String url) throws Exception {
	try {
	  return((new URL(url)).openStream());
	} catch (Exception e) {
	  if (app == null) return null;
	}
	try {
	  return((new URL(app.getCodeBase(), url)).openStream());
	} catch (Exception e) { }
	try {
	  return((new URL(app.getDocumentBase(), url)).openStream());
	} catch (Exception e) {
	  return(null);
	}
  }
}
